/* $Id: xilisf.h,v 1.2 2007/02/23 08:20:30 mta Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xilisf.h
*
* The Xilinx In-System-Flash(ISF) Library supports the Xilinx In System Flash
* hardware. The library is verified on the S3AN board.
*
* <b>Library Description</b>
*
* The library enables higher layer software (e.g. an application) to
* communicate with the ISF.
*
* The library allows the user to write, read and erase the ISF. The user can
* also protect the data stored in the ISF from unwarranted modification by
* enabling the Sector protection feature.
*
* All the API's in the library are asynchronous. The transfer is initiated and
* the control is given back to the user application. The user application has to
* keep track of whether the initiated operation is completed successfully.
*
* The library supports only interrupt driven mode. It is the application's
* responsibility to acknowledge the associated Interrupt Controller interrupts.
*
* <b>Device Operation</b>
*
* The ISF operates as a slave device on the SPI bus, with Xilinx SPI core
* operating as the Master. The library uses XSPI driver for communicating with
* ISF.
*
* <b>Device Geometry</b>
*
* The ISF is divided into Sectors. Each Sector consists of multiple Blocks. Each
* Block contains multiple Pages. Each Page contains multiple Bytes. The Number
* of Sectors, Blocks Per Sector, Pages Per Block and Bytes Per Page varies
* for different ISF's. The ISF used can be known by reading the Device ID
* present in Status Register.
*
* There are two addressing mode supported by the device:
*	- Default-Addressing Mode
*	- Power-Of-2 Addressing mode.
*
* In Default Addressing mode the ISF contains 3% more Addressing memory than the
* Power-Of-2 addressing mode. By default the ISF in the Xilinx devices are
* delivered with Default-Addressing mode. The addressing mode of the ISF can be
* known by reading the Device Status Register.
*
* <b>Library Initialization</b>
*
* The function call XIsf_Initialize() should be called by the application before
* any other function in the library. This function will fetch the device code
* and determine the geometry of the ISF used.
*
* <b>Write Operation</b>
*
* The write call can be used to write a minimum and maximum of one page of
* data. If an attempt is made to write more than one Page of data at a time,
* the write call returns an error. Once the user initiates a write operation,
* the ISF takes time to complete the write operation internally. The user has
* to read the status register to know if the ISF is still busy with previous
* initiated operation before initiating a new one.
*
* <b>Read Operation</b>
*
* The read call can be used to read a minimum of one byte and maximum of entire
* array of flash. The user has to provide a pointer to the pre allocated buffer
* where to place the data read from the ISF.
*
* <b>Erase Operation</b>
*
* The erase operations are provided to erase a Page or a Block or a Sector. The
* user can choose according to the requirement. Once the user initiates an
* erase operation, the ISF takes time to complete the Erase operation
* internally. The user has to read the status register to know if the ISF is
* still busy with previous initiated operation before initiating a new one.
*
* <b>Sector Protection</b>
*
* The ISF is divided into sectors. Each sector can be protected individually
* from unwarranted writing/erasing. The function call XIsf_SprRead() can be
* used to know the current Sector Protection Register(SPR) content. The SPR
* can be written using XIsf_SprProgram(). The SPR should be erased before
* writing. This can be done using XIsf_SprErase().
*
* Once the SPR is written successfully, the Sector Protection should be enabled
* explicitly. This can be done using XIsf_SpEnable() call. Similarly the Sector
* Protection can be disabled using XIsf_SpDisable().
*
* <b>ISF Information Read</b>
*
* The ISF information can be read by using the XIsf_GetDeviceInfo() call. This
* provides the information such as Manufacturer ID, Vendor-specific device
* family identifier, The vendor-specific device identifier for the specified
* family, The number of bits stored per memory cell, The product version and
* The number of additional Extended Device Information bytes. Read the
* Spartan-3AN In-System Flash User Guide for more information.
*
* The device status register information can be obtained using XIsf_GetStatus().
*
* @note
*
* This library is intended to be RTOS and processor independent. It works with
* physical addresses only. Any needs for dynamic memory management, threads,
* mutual exclusion, virtual memory or cache control must be satisfied by the
* layer above this driver.
*
* <pre>
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a mta  07/15/07 First release
*
* </pre>
*
******************************************************************************/
#ifndef XILISF_H /* prevent circular inclusions */
#define XILISF_H /* by using protection macros */

#ifdef __cplusplus
extern "C" {
#endif

/***************************** Include Files *********************************/

#include "xspi.h"

/************************** Constant Definitions *****************************/

/**
 * Definitions of Read commands.
 */
#define XISF_COMMAND_RANDOM_READ		0x03 /* Random read command */
#define XISF_COMMAND_ISFINFO_READ		0x1F /* Device info command */

/**
 * Definitions of Write commands.
 */
#define XISF_COMMAND_PAGEPROGRAM_WRITE	 	0x82 /* Page Program command */
#define XISF_COMMAND_AUTOPAGE_WRITE		0x58 /* Auto write command */


/**
 * Definitions of Erase commands.
 */
#define XISF_COMMAND_PAGE_ERASE			0x81 /* Page Erase command */
#define XISF_COMMAND_BLOCK_ERASE		0x50 /* Block erase command */
#define XISF_COMMAND_SECTOR_ERASE		0x7C /* Sector Erase command */


#define XISF_COMMAND_STATUSREG_READ		0xD7 /* Status read command */

/**
 * Definitions of commands used for
 *  - Erasing SPR
 *  - Programming SPR
 *  - Enabling/Disabling SPR.
 */
#define XISF_COMMAND_SPR_BYTE1			0x3D /* SPR command byte1 */
#define XISF_COMMAND_SPR_BYTE2			0x2A /* SPR command byte2 */
#define XISF_COMMAND_SPR_BYTE3			0x7F /* SPR command byte3 */
#define XISF_COMMAND_SPR_BYTE4_ERASE		0xCF /* SPR erase command */
#define XISF_COMMAND_SPR_BYTE4_PROGRAM		0xFC /* SPR program command */
#define XISF_COMMAND_SPR_BYTE4_ENABLE		0xA9 /* SPR enable command */
#define XISF_COMMAND_SPR_BYTE4_DISABLE		0x9A /* SPR disable command */

/**
 * The following definitions specify the code for different devices
 * (ISF Memory sizes).
 */
#define XISF_DEVICE_XC3S50AN			0x3 /* Device ID for 3S50AN */
#define XISF_DEVICE_XC3S200AN			0x7 /* Device ID for 3S200AN */
#define XISF_DEVICE_XC3S400AN			0x7 /* Device ID for 3S400AN */
#define XISF_DEVICE_XC3S700AN			0x9 /* Device ID for 3S700AN */
#define XISF_DEVICE_XC3S1400AN			0xB /* Device ID for 3S140AN */

/**
 * Definitions of S3AN device geomtry.
 */
#define XISF_PAGES_PER_BLOCK			8  /* Pages per Block */
#define XISF_NUM_BLOCKS16_PER_SECTOR		16 /* 16 Blocks per Sector */
#define XISF_NUM_BLOCKS32_PER_SECTOR		32 /* 32 Blocks per Sector */
#define XISF_NUM_OF_SECTORS4			4  /* Four Sectors */
#define XISF_NUM_OF_SECTORS8			8  /* Eight Sector */
#define XISF_NUM_OF_SECTORS16			16 /* Sixteen Sectors */

/**
 * Definitions of command used for reading SPR.
 */
#define XISF_COMMAND_SPR_READ			0x32 /* SPR read command */

/**
 * This definitions specify the EXTRA bytes in each of the Read command
 * transaction. This count includes Command byte, address bytes and any
 * don't care bytes needed.
 */
#define XISF_RANDOM_READ_EXTRA_BYTES		4 /* Random read extra bytes */

/**
 * The following definitions specify the total bytes in Status register read
 * command transaction. This count includes Command byte and any don't care
 * bytes needed.
 */
#define XISF_STATUS_READ_BYTES			2 /* Status read bytes count */

#define XISF_ISFINFO_READ_BYTES			5 /* ISF Info read bytes count */

/**
 * The following definitions specify the EXTRA bytes in each of the Write
 * command transaction. This count includes Command byte, address bytes and
 * any don't care bytes needed.
 */
#define XISF_PAGE_WRITE_EXTRA_BYTES		4 /* Page write extra bytes */
#define XISF_PAGE_AUTO_WRITE_BYTES		4 /* Auto write extra bytes */

/**
 * The following definitions specify the EXTRA bytes in each of the Erase
 * command transaction. This count includes Command byte, address bytes and any
 * don't care bytes needed.
 */
#define XISF_ISF_PAGE_ERASE_BYTES		4 /* Page erase extra bytes */
#define XISF_ISF_BLOCK_ERASE_BYTES		4 /* Block erase extra bytes */
#define XISF_ISF_SECTOR_ERASE_BYTES		4 /* Sector erase extra bytes */

/**
 * The following definitions specify the EXTRA bytes in each of the commands
 * operating on SPR. This count includes Command byte and any don't care bytes
 * needed.
 */
#define XISF_SPR_ERASE_BYTES			4 /* SPR erase extra bytes */
#define XISF_SPR_RDWR_BYTES			4 /* SPR RD/WR extra bytes */

#define XISF_ISFINFO_EXTRA_BYTES		1 /* ISF info extra bytes */

/**
 * The following definitions specify the status register bit definitions.
 */
#define XISF_FLASH_SR_ADDR_MODE_MASK		0x01 /* Address mode mask */
#define XISF_FLASH_SR_COMPARE_MASK		0x40 /* Compare mask */
#define XISF_FLASH_SR_IS_READY_MASK		0x80 /* Ready mask */
#define XISF_FLASH_SR_DEVICEID_MASK		0x2C /* Device ID mask */

#define XISF_FLASH_SR_DEVICEID_SHIFT_MASK	2 /* Device ID shift mask */

/**
 * The following definitions determine the addressing mode of ISF.
 */
#define XISF_FLASH_POWEROFTWO_ADDRESS		0x01 /* Pow-Of-2 address mask */
#define XISF_FLASH_DEFAULT_ADDRESS		0x00 /* Default address mask */

#define XISF_DUMMYBYTE				0xff /* Dummy byte to fill */

#define XISF_SECTORADDR_BYTE_SHIFT_MASK		1 /* Sector byte shift mask */
#define XISF_BLOCKADDR_BYTE1_SHIFT_MASK		7 /* Block byte1 shift mask */
#define XISF_BLOCKADDR_BYTE2_SHIFT_MASK		4 /* Block byte2 shift mask */
#define XISF_PAGEADDR_BYTE2_SHIFT_MASK		1 /* Page byte1 shift mask */
#define XISF_BYTEADDR_BYTE2_SHIFT_MASK		8 /* Page byte2 shift mask */

#define XISF_BUFFER_SIZE			540 /* Write Buffer size */

/*
 * Byte Positions.
 */
#define BYTE0					0 /* Byte 0 position */
#define BYTE1					1 /* Byte 1 position */
#define BYTE2					2 /* Byte 2 position */
#define BYTE3					3 /* Byte 3 position */
#define BYTE4					4 /* Byte 4 position */
#define BYTE5					5 /* Byte 5 position */

/**************************** Type Definitions *******************************/

/**
 * The following definition specify the address in the ISF memory.
 */
typedef struct {
	u8 SectorAddr; 	/**< Sector number in the ISF. */
	u8 BlockAddr; 	/**< Block number in the Sector. */
	u8 PageAddr; 	/**< Page number in the Block. */
	u16 ByteAddr; 	/**< Byte address in the Page. */
}XIsf_Address ;

struct IsfConfig {
	u16 BytesPerPage;	/**< Number of Bytes per Page. */
	u16 PagesPerBlock;	/**< Number of pages per block. */
	u16 BlocksPerSector; 	/**< Number of Blocks per Sector. */
	u16 NumOfSectors; 	/**< Number of sectors in ISF. */
	u16 NumOfBlocks; 	/**< Number of Blocks in ISF. */
	XSpi *IsfSpiInstance;	/**< SPI Device Instance pointer. */
	u32 IsfSlaveSelect;	/**< SPI Slave select for the ISF */
	u8 IsfWriteBuffer[XISF_BUFFER_SIZE]; /**< Write Buffer. */
	u8 IsReady;
};

/************************** Variable Definitions *****************************/

/**
 * Instance of IsfConfig. There will be only one instance of IsfConfig per
 * Library.
 */
extern struct IsfConfig IsfDev;

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

/*
 * Misc Functions.
 */
int XIsf_Initialize(XSpi *Spi, u32 SlaveSel);

/*
 * Function to read the Status Register.
 */
int XIsf_GetStatus(u8 *ReadPtr);

/*
 * Functions to read the ISF information.
 */
int XIsf_GetDeviceInfo(u8 *ReadPtr);

/*
 * Functions for Reading the ISF buffer.
 */
int XIsf_Read(XIsf_Address *Addr, u8 *ReadPtr, u32 ByteCount);

/*
 * Functions for Writing the ISF buffer.
 */
int XIsf_Write(XIsf_Address *Addr, u8 *BufferPtr, u32 ByteCount);

int XIsf_AutoPageWrite(XIsf_Address *Addr);

/*
 * Functions for Erasing the ISF.
 */
int XIsf_PageErase(XIsf_Address *Addr);

int XIsf_BlockErase(XIsf_Address *Addr);

int XIsf_SectorErase(XIsf_Address *Addr);

/*
 * Functions related to Sector protection.
 */
int XIsf_SprErase(void);

int XIsf_SprRead(u8 *ReadPtr);

int XIsf_SprProgram(u8 *BufferPtr);

int XIsf_SpEnable(void);

int XIsf_SpDisable(void);

#ifdef __cplusplus
}
#endif

#endif  /* end of protection macro */

