/* $Id: xilisf.c 1.1 2006/02/17 21:21:36 mta Exp $ */
/******************************************************************************
*
*       XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"
*       AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND
*       SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,
*       OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,
*       APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION
*       THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,
*       AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE
*       FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY
*       WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE
*       IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR
*       REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF
*       INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
*       FOR A PARTICULAR PURPOSE.
*
*       (c) Copyright 2007 Xilinx Inc.
*       All rights reserved.
*
******************************************************************************/
/*****************************************************************************/
/**
*
* @file xilisf.c
*
* This file contains the library functions to initialize and read the
* device information of the In System Flash(ISF). Refer xilisf.h for detailed
* description.
*
* <pre>
*
* MODIFICATION HISTORY:
*
* Ver   Who  Date     Changes
* ----- ---- -------- -----------------------------------------------
* 1.00a mta  07/11/07 First release
*
* </pre>
*
******************************************************************************/

/***************************** Include Files *********************************/

#include "includes/xilisf.h"

/************************** Constant Definitions *****************************/

/**************************** Type Definitions *******************************/

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

int IsfTransfer(u8 *WritePtr, u8* ReadPtr,u32 ByteCount);

/************************** Variable Definitions *****************************/

/*
 * ISF device geometry.
 */
typedef struct {
	u8 DeviceCode;      /* Device code.*/
	u8 PagesPerBlock;   /* Number of Pages per Block. */
	u8 BlocksPerSector; /* Number of Blocks per sector. */
	u8 NumOfSectors;    /* Number of sectors in the device. */
}IsfDeviceGeometry;

static IsfDeviceGeometry IsfDevices[] = {
	{XISF_DEVICE_XC3S50AN, XISF_PAGES_PER_BLOCK,
			XISF_NUM_BLOCKS16_PER_SECTOR, XISF_NUM_OF_SECTORS4},
	{XISF_DEVICE_XC3S200AN, XISF_PAGES_PER_BLOCK,
			XISF_NUM_BLOCKS32_PER_SECTOR, XISF_NUM_OF_SECTORS8},
	{XISF_DEVICE_XC3S700AN, XISF_PAGES_PER_BLOCK,
			XISF_NUM_BLOCKS32_PER_SECTOR, XISF_NUM_OF_SECTORS16},
	{XISF_DEVICE_XC3S1400AN, XISF_PAGES_PER_BLOCK,
			XISF_NUM_BLOCKS32_PER_SECTOR, XISF_NUM_OF_SECTORS16}
};

struct IsfConfig IsfDev;

/************************** Function Definitions ******************************/

/*****************************************************************************/
/**
*
* The ISF device status register is read and the geometry of the underlying
* device is determined. This function should be called before any other function.
*
* @param	Spi is a pointer to the XSpi instance to be worked on.
* @param	SlaveSel is a 32-bit mask with a 1 in the bit position of the
*		slave being selected. Only one slave can be selected. The
*		SlaveMask can be zero if the slave is being deselected.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		The XIsf_Initialize() function is blocking call.
*
******************************************************************************/
int XIsf_Initialize(XSpi *Spi, u32 SlaveSel)
{
	int Status;
	u8 DeviceCode;
	u8 AddrMode;
	u32 Index;
	u8 ReadBuf[2];
	u8 StatusRegister;
	u8 NumOfDevices;

	IsfDev.IsReady = FALSE;

	if(Spi == NULL) {
		return XST_FAILURE;
	}

	if(SlaveSel == 0) {
		return XST_FAILURE;
	}

	/*
	 * Check if SPI has started.
	 */
	 if(Spi->IsStarted != XCOMPONENT_IS_STARTED) {
		 return XST_FAILURE;
	 }

	IsfDev.IsfSpiInstance = Spi;
	IsfDev.IsfSlaveSelect = SlaveSel;

	/*
	 * The IsReady is made TRUE to fetch the Status Register content.
	 */


	/*
	 * Get the status register content.
	 * The IsReady is temporarily made TRUE to fetch the Status Register
	 * content.
	 */
	IsfDev.IsReady = TRUE;
	Status = XIsf_GetStatus(ReadBuf);
	IsfDev.IsReady = FALSE;
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}



	do {
		Status = XSpi_SetSlaveSelect(IsfDev.IsfSpiInstance,
							IsfDev.IsfSlaveSelect);
	} while(Status == XST_FAILURE);

	StatusRegister = ReadBuf[BYTE1];

	/*
	 * Get the device code from the status register.
	 */
	DeviceCode = (StatusRegister & XISF_FLASH_SR_DEVICEID_MASK) >>
					XISF_FLASH_SR_DEVICEID_SHIFT_MASK;

	/*
	 * Get the Address mode from the status register.
	 */
	AddrMode = StatusRegister & XISF_FLASH_SR_ADDR_MODE_MASK;

	NumOfDevices = sizeof(IsfDevices) / sizeof(IsfDeviceGeometry);
	for(Index = 0; Index < NumOfDevices; Index++) {
		if(DeviceCode == IsfDevices[Index].DeviceCode) {
			if(DeviceCode == XISF_DEVICE_XC3S1400AN)
			{
				if(AddrMode == XISF_FLASH_DEFAULT_ADDRESS) {
					IsfDev.BytesPerPage = 528;
				} else {
					IsfDev.BytesPerPage = 512;
				}
			} else {
				if(AddrMode == XISF_FLASH_DEFAULT_ADDRESS) {
					IsfDev.BytesPerPage = 264;
				} else {
					IsfDev.BytesPerPage = 256;
				}

				IsfDev.PagesPerBlock =
						IsfDevices[Index].PagesPerBlock;

				IsfDev.BlocksPerSector =
					IsfDevices[Index].BlocksPerSector;

				IsfDev.NumOfSectors =
						IsfDevices[Index].NumOfSectors;
			}

			IsfDev.IsReady = TRUE;
		}
	}

	/*
	 * If the device is not supported, return Failure.
	 */
	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function reads the ISF Status register.
*
* @param	ReadPtr is a pointer to the memory where the Status Register
*		content is copied.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		The status register content is stored at the second byte pointed
*		by the ReadPtr.
*
******************************************************************************/
int XIsf_GetStatus(u8 *ReadPtr)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(ReadPtr == NULL) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_STATUSREG_READ;
	IsfDev.IsfWriteBuffer[BYTE1] = XISF_DUMMYBYTE;

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, ReadPtr,
						XISF_STATUS_READ_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function reads the ISF Device information.
*
* @param	ReadPtr is a pointer to the memory where the Device information
*		is copied.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		The Device information is stored at the second byte pointed
*		by the ReadPtr.
*
******************************************************************************/
int XIsf_GetDeviceInfo(u8 *ReadPtr)
{
	int Status;

	if(IsfDev.IsReady != TRUE) {
		return XST_FAILURE;
	}

	if(ReadPtr == NULL) {
		return XST_FAILURE;
	}

	/*
	 * Prepare the Write Buffer.
	 */
	IsfDev.IsfWriteBuffer[BYTE0] = XISF_COMMAND_ISFINFO_READ;
	IsfDev.IsfWriteBuffer[BYTE1] = XISF_DUMMYBYTE;
	IsfDev.IsfWriteBuffer[BYTE2] = XISF_DUMMYBYTE;
	IsfDev.IsfWriteBuffer[BYTE3] = XISF_DUMMYBYTE;
	IsfDev.IsfWriteBuffer[BYTE4] = XISF_DUMMYBYTE;

	/*
	 * Initiate the Transfer.
	 */
	Status = IsfTransfer(IsfDev.IsfWriteBuffer, ReadPtr,
						XISF_ISFINFO_READ_BYTES);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
*
* This function configures the SPI options and performs the SPI transfer.
*
* @param	ReadPtr is a pointer to the memory where the Data read from the
*		ISF is copied.
* @param	BufferPtr is the pointer to the data to be written into ISF.
* @param	ByteCount is the number of bytes need to read/written from the
*		ISF.
*
* @return	XST_SUCCESS if successful else XST_FAILURE.
*
* @note		None.
*
******************************************************************************/
int IsfTransfer(u8 *WritePtr, u8* ReadPtr,u32 ByteCount)
{
	int Status;

	/*
	 * Set the Spi device as a master.
	 */
	Status = XSpi_SetOptions(IsfDev.IsfSpiInstance, XSP_MASTER_OPTION |
						XSP_MANUAL_SSELECT_OPTION);
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Select the ISF Slave.
	 */
	Status = XSpi_SetSlaveSelect(IsfDev.IsfSpiInstance,
							IsfDev.IsfSlaveSelect);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	/*
	 * Start the transfer.
	 */
	Status = XSpi_Transfer(IsfDev.IsfSpiInstance, WritePtr, ReadPtr,
								ByteCount);
	if(Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	return XST_SUCCESS;
}
